<?php

/*

Simple PHP class to create 'pocketmod' style booklets from map sources.

http://www.aaronland.info/php/mmPDF/
Copyright (c) 2007 Aaron Straup Cope.

This is free software, you may use it and distribute it under the
same terms as Perl itself.

$Id: mmPDF.php,v 1.26 2007/08/13 15:42:29 asc Exp $

*/

	require_once("pmPDF.php");
	require_once("qr.php");

	class mmPDF extends pmPDF {

			function mmPDF($args=array()){

				$this->_places = array();
				$this->_page = ($args['no_cover']) ? 1 : 2;
				$this->_appid = $args['appid'];
				$this->_qr = $args['qr'];
				$this->_mm = $args['modestmaps'];
				$this->_singlemap = $args['simple'];
				$this->_src = $args['source'];

				$this->_index = array();
				$this->pmPDF($args);
			}
			
			function add_map($url, $max=0){

				if (preg_match("/platial\.com/", $url)){
					return $this->add_platial_map($url, $max);
				}

				if (preg_match("/google\.com/", $url)){
					return $this->add_google_map($url, $max);
				}

				if (preg_match("/del\.icio\.us/", $url)){
					return $this->add_delicious_map($url, $max);
				}

				if (preg_match("/delicious\.com/", $url)){
					return $this->add_delicious_map($url, $max);
				}

				return $this->add_kml_file($url, $max);
			}
			
			/* Data-fetching functions */

			function add_delicious_map($del_url, $max=0){

				require("restobook.php");
				require("machinetag.php");

				$del_url .= "?count=100&raw=1";

				$res  = $this->simple_get($del_url);
				$json = $this->decode_json($res);

				if (! $json){
					return null;
				}
				
				$places = 0;

				foreach ($json as $post){

					$url = $post->u;
					$desc = $this->place_description($post->n, 'delicious');
					$name = $this->place_name($post->d, 'delicious');
					$tags = $post->t;
					$geo = array();

					/* To do : abstractify.... */

					$display_tags = array();

					foreach ($tags as $t){

						$mt = new MachineTag($t);

						if (! $mt->is_machinetag()){
							$display_tags[] = $t;
							continue;
						}

						if ($mt->namespace() != 'geo'){
							continue;
						}

						$pred = $mt->predicate();

						if (preg_match("/(?:lat|lon)/", $pred)){
							$geo[$pred] = $mt->value();
						}

					}

					$extra = array();

					if ($this->_qr){

						$extra['qr'] = array();
						
						if ($desc){
							$extra['qr'][] = array('body' => $desc, 'color'=>array(46, 139, 87));
						}

						$extra['qr'][] = array('body' => $url, 'color' => array(47, 79, 79));
						
						if (preg_match("/\/json\/([^\/]+)\//", $del_url, $m)){
							$username = $m[1];
							$edit = sprintf("http://del.icio.us/%s?v=4&noui&jump=close&url=%s", $username, urlencode($url));
							$extra['qr'][] = array('body' => $edit, 'color'=>array(0, 0, 180));
						}
					}

					/* 
					sort($display_tags);
					$desc = $desc . "\n" . implode(",", $display_tags);
					*/

					if ($this->add_place($name, $desc, $geo['lat'], $geo['lon'], $extra)){
						$places++;
					}

					if (($max) && ($places >= $max)){
						break;
					}
				}

				return $count;
			}

			function add_platial_map($url, $max=0){
				$url = str_replace("kmlnl", "kml", $url);
				return $this->add_kml_file($url, $max, "platial");				
			}

			function add_google_map($url, $max=0){
				# please for you to properly parse the URL...
				$url = str_replace("output=nl", "output=kml", $url);
				$url = str_replace("&mpnum=3", "", $url);
				return $this->add_kml_file($url, $max, 'google');
			}

			function add_kml_file($kml, $max=0, $provider="google"){

				$xml = simplexml_load_file($kml);
				
				if (! $xml){
					return null;
				}

				$places = 0;

				foreach ($xml->Document->Placemark as $pl){

					$name = $this->place_name($pl->name, $provider);
					$desc = $this->place_description($pl->description, $provider);

					list($lon, $lat) = explode(",", $pl->Point->coordinates);

					if ($this->add_place($name, $desc, $lat, $lon)){
						$places ++;
					}

					if (($max) && ($places >= $max)){
						break;
					}
				}
					
				return $places;
			}

			/* Data-massaging functions */

			function place_name($name, $provider="google"){
				return trim($name);
			}

			function place_description($desc, $provider="google"){

				if ($provider=='delicious'){
					$desc = html_entity_decode($desc);

					$rb = new Restobook();

					if ($rb->parse($desc)){
						$desc = $rb->get("address", "raw");

						if ($ph = $rb->get("phone", "raw")){
							$desc .= " / {$ph}";
						}
					}
				}

				else if ($provider=='platial'){
					$desc = str_replace("<br>", "\n", $desc);
					preg_match("/<div style='color:#666'>([^<]+)<\/div>/", $desc, $m);
					$desc = $m[1];
				}

				else {
					$desc = str_replace("<br>", "\n", $desc);
				}
				
				return trim(strip_tags($desc));
			}

			/* Actual page generating functions */

			function add_place($name, $desc, $lat, $lon, $extra=array()){

				$key = md5($name . $desc);

				if (array_key_exists($key, $this->_places)){
					continue;
				}

				$this->_places[$key] ++;

				$lat = trim($lat);
				$lon = trim($lon);

				error_log("add place : $lat $lon");
					
				if (! $this->_singlemap){

					while ($this->_page % 2){
						$this->_page ++;
					}

					$map1 = $this->get_img($lat, $lon, "city");
					$res = $this->add_image($map1, $this->_page);

					if ($extra['qr']){
						$this->add_qr($extra['qr']);
					}

					$this->_page = $res['page'] + 1;
					unlink($map1);
				}

				$map2 = $this->get_img($lat, $lon, "street");
				
				$res = $this->add_image($map2, $this->_page);
				$this->_page = $res['page'];

				$txt = "{$name}\n{$desc}";
				$this->add_text($txt, $this->_page, array('y' => 2.5, 'resume' => false));

				# 

				if (! array_key_exists($name, $this->_index)){
					$this->_index[$name] = array();
				}

				$this->_index[$name][] = $this->_page;
				$this->_page = $res['page'] + 1;

				# 

				unlink($map2);
				sleep(1);

				return 1;
			}

			function add_index(&$idx){

				ksort($idx);

				# 
				# how many pages (a) remain blank given the total number of sheets
				# what is the page number (b) of the last page
				# how many pages (c) will the index use
				# if c < b then set the start page to b - c
				# otherwise start from the current offset
				#

				$page_use = $this->calculate_page_use_for_idx($idx);

				$keys   = array_keys($this->_buffer);
				# fix me - check for no cover...
				$cnt = (count($keys) + 1) + $page_use;
				$sheets = ceil(($cnt + 1) / 8);
				$last   = $sheets * 8;

				if (($this->_page + $page_use) < $last){
					$this->_page = $last - ($page_use - 1);
				}

				$args = array('resume' => false);

				foreach ($idx as $n => $pgs){

					$n .= " ";

					foreach ($pgs as $p){

						$nlen = $this->GetStringWidth($n);
						$plen = $this->GetStringWidth(" {$p}");				      

						$sect = $this->_sect_width;
						$limit = $sect - $plen;

						$str_len = ($nlen + $plen);
					
						while ($limit > ($str_len + $plen)){

							$n .= ".";
							$nlen = $this->GetStringWidth($n);
							$str_len = ($nlen + " " + $plen);
						}

						$res = $this->add_text("{$n} {$p}", $this->_page, $args);
						$this->_page = $res['page'];

						$args = array();
					}
				}

				return 1;
			}

			function Output($name='', $dest=''){
				$this->add_index($this->_index);
				parent::Output($name, $dest);
				return;
			}

			/* QR functions */

			function mk_qr($body, $colour=array(0, 0, 0)){

				$qr_path = $this->tempfile("qr" . md5($body));
				$qr = new QR($this->_qr);

				$args = array('d' => $body, 'path' => $qr_path, 'color' => $colour);
				$qr->encode($args);
				return $qr_path;
			}

			function add_qr(&$qr_data){

				$coords = array('y' => 2.5, 'h' => .75);

				foreach ($qr_data as $data){
					$barcode = $this->mk_qr($data['body'], $data['color']);
					$this->add_image($barcode, $this->_page, $coords);
					$coords['x'] += .75;
				}
			}
			
			/* Image fetching functions */

			function get_img($lat, $lon, $zoom_label){

				$zoom = $this->mk_zoom($zoom_label);

				if ($this->_src == 'modestmaps'){
					return $this->get_img_modestmaps($lat, $lon, $zoom);
				}

				return $this->get_img_yahoo_imageapi($lat, $lon, $zoom);
			}

			function get_img_modestmaps($lat, $lon, $zoom){

				$fmt = "%s?provider=%s&marker=%s&latitude=%s&longitude=%s&accuracy=%s&height=%s&width=%s";

				$url = sprintf($fmt, $this->_mm['server'], $this->_mm['provider'], $this->_mm['marker'], $lat, $lon, $zoom, 500, 500);
				return $this->getstore($url, $zoom);
			}

			function get_img_yahoo_imageapi($lat, $lon, $zoom){
				
				$req = "http://api.local.yahoo.com/MapsService/V1/mapImage?appid={$this->_appid}&latitude={$lat}&longitude={$lon}&zoom={$zoom}&image_width=500&image_type=png";
				
				$res = $this->simple_get($req);
				$xml = new SimpleXMLElement($res);
	
				if (! $xml){
					return;
				}

				return $this->getstore($xml, $zoom);
			}

			/* Index functions - to be moved into 'pmPDF.php' (once they don't suck) */

			function calculate_page_use_for_idx(&$idx){

				$txt = '';

				foreach ($idx as $n => $pgs){

					$n .= " ";

					foreach ($pgs as $p){

						$nlen = $this->GetStringWidth($n);
						$plen = $this->GetStringWidth(" {$p}");				      

						$sect = $this->_sect_width;
						$limit = $sect - $plen;

						$str_len = ($nlen + $plen);
					
						while ($limit > ($str_len + $plen)){

							$n .= ".";
							$nlen = $this->GetStringWidth($n);
							$str_len = ($nlen + " " + $plen);
						}

						$txt .= "{$n} {$p}\n";
					}
				}
				
				return $this->calculate_page_use($txt);
			}

			function calculate_page_use($txt){

				$start_page = $this->_page;

				$args = array('calc_only' => true);
				$res = $this->add_text($txt, $start_page, $args);

				$this->_page = $start_page;

				$num = ($res['page'] - $start_page) + 1;
				return $num;
			}

			/* Utility functions */

			function getstore($url, $suffix){
				$tmp = $this->tempfile($suffix);

				$fh  = fopen($tmp, "w");
				fwrite($fh, $this->simple_get($url));
				fclose($fh);
				
				return $tmp;
			}

			function decode_json($json){

				json_decode($json);				
				
				if (is_array($json)){
					return $json;
				}

				include('JSON.php');
				$js = new Services_JSON();
				$json = $js->decode($json);

				if (is_array($json)){
					return $json;
				}

				return null;
			}

			function tempfile($suffix, $ext="png"){
				return tempnam("/tmp", time()) . "-{$suffix}.{$ext}";
			}

			function mk_zoom($label='street'){
				if ($this->_src=='modestmaps'){
					return $this->mk_zoom_modestmaps($label);
				}

				return $this->mk_zoom_yahoo($label);
			}

			function mk_zoom_yahoo($label='street'){
				if ($label=='city'){
					return 4;
				}
				
				return 1;
			}
			
			function mk_zoom_modestmaps($label){

				if ($label=='city'){
					return 14;
				}

				return 17;
			}
	}
?>